﻿/* MVRNavigationController
 * MiddleVR
 * (c) MiddleVR
 */

using MiddleVR;
using UnityEngine;

[RequireComponent(typeof(CharacterController))]
public class MVRNavigationController : MonoBehaviour
{
    public string MovementDirectionNodeName = "HandNode";
    public float HeadProtectionRadius = 0.2f;
    public float WalkSpeed = 1.0f;
    public float RotationSpeed = 100.0f;
    public bool Straffe = false;
    public float GravityMultiplier = 10.0f;

    private GameObject _head;
    private GameObject _movementDirectionNode;
    private CharacterController _characterController;

    private CollisionFlags _collisionFlags;
    private Vector2 _input;
    private Vector3 _movement = Vector3.zero;

    private void Start()
    {
        _characterController = this.GetComponent<CharacterController>();
    }

    private void Update()
    {
        if (_head == null)
        {
            _head = GameObject.Find("HeadNode");
            return;
        }

        _characterController.height = HeadProtectionRadius + (_head.transform.position.y - this.transform.position.y);

        Vector3 center = this.transform.InverseTransformVector(_head.transform.position - this.transform.position);
        center.y = _characterController.height / 2.0f;

        Vector3 headMovement = center - _characterController.center;
        headMovement.y = 0.0f;

        _characterController.center = center;

        this.transform.position -= headMovement;
        _characterController.Move(headMovement);

        // Follow ground
        if (_characterController.isGrounded)
        {
            _movement.y = 0f;
        }
        else
        {
            _movement.y = 0f;
        }
    }

    private void FixedUpdate()
    {
        float speed;
        GetInput(out speed);

        if (_movementDirectionNode == null)
        {
            _movementDirectionNode = GameObject.Find(MovementDirectionNodeName);
            return;
        }

        Vector3 movementForward = _movementDirectionNode.transform.forward;
        Vector3 movementRight = _movementDirectionNode.transform.right;

        // No fly for the moment
        movementForward.y = 0.0f;
        movementForward.Normalize();
        movementRight.y = 0.0f;
        movementRight.Normalize();

        Vector3 desiredMove;
        if (Straffe)
        {
            desiredMove = movementForward * _input.y + movementRight * _input.x;
        }
        else
        {
            desiredMove = movementForward * _input.y;
        }

        _movement.x = desiredMove.x * speed;
        _movement.z = desiredMove.z * speed;


        if (!_characterController.isGrounded)
        {
            _movement += Physics.gravity * GravityMultiplier * Time.fixedDeltaTime;
        }

        _collisionFlags = _characterController.Move(_movement * Time.fixedDeltaTime);

        if (!Straffe)
        {
            this.transform.RotateAround(_head.transform.position, Vector3.up, _input.x * RotationSpeed * Time.fixedDeltaTime);
        }
    }

    private void GetInput(out float speed)
    {
        // Read input
        float horizontal = MVR.DeviceMgr.GetWandHorizontalAxisValue();
        float vertical = MVR.DeviceMgr.GetWandVerticalAxisValue();

        speed = WalkSpeed;
        _input = new Vector2(horizontal, vertical);

        // normalize input if it exceeds 1 in combined length:
        if (_input.sqrMagnitude > 1)
        {
            _input.Normalize();
        }
    }

    private void OnControllerColliderHit(ControllerColliderHit hit)
    {
        Rigidbody body = hit.collider.attachedRigidbody;

        if (_collisionFlags == CollisionFlags.Below)
        {
            return;
        }

        if (body == null || body.isKinematic)
        {
            return;
        }

        body.AddForceAtPosition(_characterController.velocity * 0.1f, hit.point, ForceMode.Impulse);
    }
}
