﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using MiddleVR.Unity;

public class ServerCanvasGUIScript : MonoBehaviour
{
    public GameObject Chair = null;
    private Material m_Material = null;
    private Color m_OriginalColor = Color.white;

    void Start()
    {
        Cluster.AddMessageHandler<bool>(this, ClusterOnButtonPress, channel: 1);
        Cluster.AddMessageHandler<float>(this, ClusterOnSliderChanged, channel: 2);

        if(Chair != null)
        {
            m_Material = Chair.GetComponent<Renderer>().material;
            m_OriginalColor = m_Material.color;
        }
    }

    public void OnButtonPress()
    {
        Debug.Log("OnButtonPress");
        Cluster.BroadcastMessage(this, true, channel: 1);
    }

    public void OnSliderChanged(float iValue)
    {
        Debug.Log("Slider value: " + iValue);
        Cluster.BroadcastMessage(this, iValue, channel: 2);
    }

    private void ClusterOnButtonPress(bool iBool)
    {
        // Actually do the button press action on all cluster nodes

        // Note: This will also be called even if the configuration
        // is not a cluster configuration

        Debug.Log("ClusterOnButtonPress");
        Debug.Log("Show/Hide Chair!");

        if(Chair!=null)
        {
            Chair.SetActive(!Chair.activeSelf);
        }
    }

    private void ClusterOnSliderChanged(float iValue)
    {
        // Actually do the action

        // Note: This will also be called even if the configuration
        // is not a cluster configuration

        Debug.Log("ClusterOnSliderChanged: " + iValue);
        

        if (Chair != null)
        {
            m_Material.color = m_OriginalColor + iValue * Color.red;
        }
    }
}
